% S = correlated_gen(T, N, r, {'rate', 40}, {'lambda0', 1}, ...
%              {'sigma', 10})
%
% Makes a matrix of pairwise-correlated spike trains, in "Sillito"
% format. Each row is pairwise correlated with each of the
% others. Each row is made up of Poisson spikes at a uniform
% rate. Note that spike times are in milliseconds, but T and rate 
% are in secs and spikes/sec. 
%
% RETURNS:
% --------
%
%  -S     A matrix in "Sillito" format (see Contents.m)
%
%
% PARAMETERS:
% -----------
%
%  -T     total time, in secs, for which to generate spikes.
%
%  -N     number of spike trains to generate
%
%  -r     correlation coefficient between two cells for the total
%         number of spikes fired. range is [0, 1]. The closer to
%         1, the stronegr the cross-correlogram peak.
%
%
% OPTIONAL PARAMETERS:
% --------------------
%
%  -rate   spikes/sec for final spike trains; default is 40.
%
%  -sigma  magnitude, in ms, of jitter in shared spikes. This
%          controls the width of the resulting cross-correlogram
%          peak
%
%
% EXAMPLES:
% ---------
%
%  generate 2 spike trains for 3 seconds with a correlation
%  coefficient of 0.2 and then plot a raster:
% 
%  >> S = correlated_gen(3, 2, 0.2); nrastermaker(S);
%
%
%  Now generate the same, but instead of the default 10 ms
%  cross-correlation, make them both very highly correlated and
%  with very tight timing; the two spike trains will look the same.
%
%  >> S = correlated_gen(3, 2, 0.99, 'sigma', 1); nrastermaker(S);
%
%  Finally, one with very high correlation but loose timing:
%
%  >> S = correlated_gen(3, 2, 0.99, 'sigma', 10); nrastermaker(S);
%

% written by Carlos Brody, Sep 2004



function [S,sigma] = correlated_gen(T, N, r, varargin)
   
   pairs = { ...
       'rate'         12   ; ...      
       'lambda0'       1   ; ...
       'sigma'        10   ; ...
   }; parseargs(varargin, pairs);
   
   if r==0, S = uncorrelated_gen(T, N, rate); return; end;   
   
   Npick = round(rate/lambda0);
   Npool = round(Npick/r);
   

   % --- Make the pool of Poisson trains from which we'll pick

   Spool = uncorrelated_gen(T, Npool, lambda0);
   
   % --- Now make the actual output
   
   S = zeros(N, ceil(rate*T + 4*sqrt(rate*T)));
   for i=1:rows(S),
      picked = randperm(Npool);
      picked = picked(1:Npick);
      spiketimes = [];
      for j=1:length(picked),
	 newspikes = Spool(picked(j), 2:1+Spool(picked(j),1));
	 newspikes = newspikes + sigma*randn(1,length(newspikes));
	 spiketimes = [spiketimes newspikes];
      end;
      S(i,1) = length(spiketimes);
      S(i,2:1+length(spiketimes)) = sort(spiketimes);
   end;
   
   S = S(:,1:1+max(S(:,1)));
   
   
   
   