% [x, timebase, mu, sd] = dxcorr(C1, C2)   Sillito x-correlations
%
% cross-correlation with significance limits, done a la Carlos
% variance style (through binning).
%
% [x, timebase, mu, sd, sx, stime, smu, psth1, psth2] = 
%       dxcorr(C1, C2, {'from', from}, {'to', to}
%                      {'binwidth', binwidth=6}
%                      {'xc'} {'DaveY' | 'davey'}
%                      {'HzY' | 'hzy'}
%                      {'plot' | 'noplot'}
%                      {'normdiag' | 'nodiag' | 'zerodiag'}
%                      {'nosub'}
%                      {'datacolor' datacolor='y'}
%                      {'sdcolor' sdcolor='c'}
%                      {'datalwidth', datalwidth=0.5}
%                      {'sdstyle', sdstyle='-'})
%
%
% sx and stime are like x and t, except they are ready for
% "stairs" style plotting. 
%
% PARAMETERS:
%
% C1 and C2 must be Sillito-style matrices.
%
% 'from' means ignore any spikes before this time. Default is zero.
%         Note that binning quantization may make this number
%         inexact. 
%
% 'to' means ignore any spikes after this time. Default is Inf.
%         Note that binning quantization may make this number
%         inexact. 
%
% 'binwidth' is self-explanatory. Its units are ms. nbins is 
%         set to round((to-from)/binwidth).
%
% 'xc' means do the excitability correction.
%
% 'DaveY' means multiply results by the number of rasters, to get
%         axes like those in Dave West's Xcorel program. 'davey'
%         is the same. 
%
% 'HzY'   overrides 'DaveY' and means express the y-axis in spikes
%         squared per second.
%
% 'counts' means express the y-axis in average counts per bin
%
% 'norm'   means express the y-axis in spikes/second
% 
% 'plot' is the default: conversely, 'noplot' means don't display
%         the esults 
%
% 'nodiag' means subtract from the zero bin any self-spikes
%         (intended for autocovariograms).  
%
% 'zerodiag' means make make the zero bin in the xcorr equal to its
%         expected value 
%
% 'normdiag' is the default and means don't do anything to the
%         zero bin.
%
% 'nosub' means don't substract the average value from displayed
%         results: default is to substract it.
%
% 'datacolor' and 'sdcolor' control the color of the xcorr and the
%         significance limits, respectively.
%
% 'datalwidth'  data line with (for plotting)
%
% 'sdstyle'     sd line style (for plotting). 'none' means don't
%               show the sd lines. 
%
function [ttop,xx, timebase, av, sd, sx, stime, sav, mu, lambda] = ...
       dxcorr(C1, C2, varargin)

%function [xx, timebase, av, sd, sx, stime, sav, mu, lambda] = ... %Carlos orginal
%      dxcorr(C1, C2, varargin)
    
   from = 0;
   to = -Inf;
   binwidth = 10;   %was 6 from Carlos
   do_plot = 1;
   substract_av = 1;  %was 1 from Carlos
   no_diag = 'normdiag';
   colororder = get(gca, 'ColorOrder');
   datacolor = colororder(1,:);
   %sdcolor = colororder(3,:);
   sdcolor = 'k';
   datalwidth = get(gcf, 'DefaultLineLineWidth');
   %sdlwidth = get(gcf, 'DefaultLineLineWidth');
   sdlwidth = 4;
   sdstyle = '-';
   excitability_correction = 0;
   davey = 0;
   hzy = 1;
   normmode = 'norm';  %Carlos had 'norm'
   mode = 'natural';
   nspks = 0;
   FontSize = get(gcf, 'DefaultAxesFontSize');
   zlw = 3;                   % Porportional width of zero lines
   zerocolor = -1;
   %   zerocolor = 0.3*[1 1 1];   % color of zero lines
   
   
   % ------- parsing input args -------------
   
   nmatrices = 0;
   arg = 1; while arg <= length(varargin),

      switch varargin{arg}
	 
	 case 'from',
	 if arg+1 <= length(varargin),
	    from = varargin{arg+1};
	    arg = arg+1;
	 end;
	 
	 case 'to',
	 if arg+1 <= length(varargin),
	    to = varargin{arg+1};
	    arg = arg+1;
	 end;
	 
	 case 'datacolor',
	 if arg+1 <= length(varargin),	
	    datacolor = varargin{arg+1};
	    arg = arg+1;
	 end;
	 
	 case 'zerocolor',
	 if arg+1 <= length(varargin),	
	    zerocolor = varargin{arg+1};
	    arg = arg+1;
	 end;
	 
	 case 'sdcolor',
	 if arg+1 <= length(varargin),
	    sdcolor = varargin{arg+1};
	    arg = arg+1;
	 end;
	 
	 case 'datalwidth',
	 if arg+1 <= length(varargin),
	    datalwidth = varargin{arg+1}; arg = arg+1;
	 end;
	 
	 case 'sdstyle',
	 if arg+1 <= length(varargin),
	    sdstyle = varargin{arg+1}; arg = arg+1;
	 end;
	 
	 case 'sdlwidth',
	 if arg+1 <= length(varargin),
	    sdlwidth = varargin{arg+1}; arg = arg+1;
	 end;
	 
	 case 'linewidth'
	 if arg+1 <= length(varargin),
	    datalwidth = varargin{arg+1}; 
	    sdlwidth = varargin{arg+1}; 
	    arg = arg+1;
	 end;
	 
	 
	 case 'FontSize',
	 if arg+1 <= length(varargin),
	    FontSize = varargin{arg+1}; arg = arg+1;
	 end;
	 
	 case 'binwidth',
	 if arg+1 <= length(varargin),
	    binwidth = varargin{arg+1};
	    arg = arg+1;
	 end;
	 
	 case 'plot', do_plot = 1;

	 case 'nspks', nspks = 1;
	 
	 case {'no_plot', 'noplot'}, do_plot = 0;
	    
	 case 'nosub', substract_av = 0;  %Carlos had 1 here
	    
	 case {'no_diag', 'nodiag'}, no_diag = 'nodiag';
	    
	 case {'zero_diag', 'zerodiag'}, no_diag = 'zerodiag';
	    
	 case {'norm_diag', 'normdiag'}, no_diag = 'normdiag';
	    
	 case 'xc', excitability_correction = 1;
	    
	 case 'N', mode = 'natural';
	 
	 case 'P', mode = 'poisson';
	 
	 case {'DaveY', 'davey'}, normmode = 'davey';
	    
	 case {'HzY', 'hzy'}, normmode = 'hzy';
	 
	 case {'norm'}, normmode = 'norm';
	 
	 case {'counts'}, normmode = 'counts';
	 
	 case {'normmode'},
	 if arg+1 <= length(varargin),
	    normmode = varargin{arg+1};
	    arg = arg+1;
	 end;
	 
	 otherwise
	 varargin{arg}
	 error('Didn''t understand the above parameter');
      
      end;

   arg = arg+1; end;

   if to == -Inf, to = max([maxtime(C1), maxtime(C2)]); end;
    
   if excitability_correction  &  strcmp(mode, 'natural'),
      fprintf(2, 'Sorry, haven''t figured out natural mode and\n');
      fprintf(2, 'xc mode yet-- ignoring xc\n');

      excitability_correction = 0;
   end;
   
   
   % -----------------------------
    
   nrasters = min([rows(C1), rows(C2)]);
%fprintf('before bins from= %d  to= %d \n',from, to);   
   nbins = round((to-from)/binwidth);
   if nbins <= 0,
      error('binwidth too large or something funny about times');
   end;
   binwidth = ((to+1e-5)-from)/nbins;
   
   ptimebase = from:binwidth:to;
   timebase =(-binwidth*(nbins-1)):binwidth:(binwidth*(nbins-1));
%fprintf('from= %d  to= %d nbins= %d\n',from, to, nbins); %---------------------------  I added
   [x, mu, lambda, msqu,lsqu] = zxcorr(C1, C2, nbins, from, to, 0);

   x = x./nrasters;  %-------------------
   mu = mu./nrasters;
   lambda = lambda./nrasters;
   msqu = msqu./nrasters - mu.^2;
   lsqu = lsqu./nrasters - lambda.^2;
   if nrasters > 1,
      msqu = (nrasters/(nrasters-1))*msqu;
      lsqu = (nrasters/(nrasters-1))*lsqu;
   end;
   
   if excitability_correction,
      alphavec = numspikes(C1, from, to); 
      alphavec = alphavec/mean(alphavec);

      betavec =  numspikes(C2, from, to); 
      betavec =  betavec/mean(betavec);
      
      ab = mean(alphavec.*betavec);
      aab = mean(alphavec.*alphavec.*betavec);
      abb = mean(alphavec.*betavec.*betavec);
   else
      ab = 1; aab = 1; abb = 1;
   end;
   
   
   av = ab*xcorr(mu, lambda);
   switch mode,
      case 'poisson',
      sd = av + abb*xcorr(mu,lambda.*lambda) + ...
	  aab*xcorr(mu.*mu,lambda);
      case 'natural',
      sd = xcorr(msqu, lsqu) + ...
	  xcorr(msqu, lambda.*lambda) + xcorr(mu.*mu, lsqu);
   end;
   
   hack = min(sd);
   if hack < 0  &  abs(hack) < 1e-12,      % numerical error
      sd = sd - hack;        % kill it
   end;
   sd = sqrt(sd/nrasters);
   
   switch no_diag,
      case 'nodiag'
      nsp1 = mean(numspikes(C1, from, to)); 
      nsp2 = mean(numspikes(C2, from, to)); 
      x(length(mu)) = x(length(mu)) - mean([nsp1, nsp2]); 
   
      case 'zerodiag',
      x(length(mu)) = av(length(mu));
      
      case 'normdiag',
      % do nuttin'
      
   end;
%----  mine 051120   
   %[stime, sav] = stairs(timebase, av);
   stime= timebase;
   sav= av;
   sx=x;
   
%    [stime, sx] = stairs(timebase, x);
%    if length(stime) > 1, 
%       stime = stime - (stime(2)-stime(1))/2; 
%    end;
%-----------------------------------------------------  
%---- Carlos stairs plot
%   [stime, sav] = stairs(timebase, av);  
%   [stime, sx] = stairs(timebase, x);
%   if length(stime) > 1, 
%       stime = stime - (stime(2)-stime(1))/2; 
%   end;
%--------------------------------------------------------------
   switch normmode,
      case 'davey',
      x = x*nrasters;
      sd = sd*nrasters;
      av = av*nrasters;
      sx = sx*nrasters;
      sav = sav*nrasters;
      
      case 'counts',
      
      case 'hzy',
      x = x*1000/binwidth;
      sd = sd*1000/binwidth;
      av = av*1000/binwidth;
      sx = sx*1000/binwidth;
      sav = sav*1000/binwidth;
      
      case 'norm',
      n1 = mean(numspikes(C1, from, to));
      n2 = mean(numspikes(C2, from, to));
      if (n1>0 & n2>0),
	 scalefactor = (1000/binwidth)/sqrt(n1*n2);  %-----Carlos uses this
	 %line
     %scalefactor = (1000/binwidth)/(n2);  %-----I made
      else
	 scalefactor = 1;
      end;
      
      x = x*scalefactor;  %-------
      sd = sd*scalefactor;
      av = av*scalefactor;
      sx = sx*scalefactor;
      sav = sav*scalefactor;
   end;
   %--- I added so xcorrfig2 works
   %ttop = max([max(2*sd), max(sx-sav)]);
   %ttop = max(max(sx-sav));
   %--------

   if do_plot,

      otimebase = timebase;
      if length(timebase) == 1,
	 timebase = [-binwidth/2 binwidth/2];  
	 sd = [sd sd];
	 stime = [-binwidth/2 binwidth/2];  
	 sx = [sx sx];
	 sav = [sav sav];
      end;
      
      if substract_av,
	 hl = plot(timebase, 2*sd, 'c', timebase, -2*sd, 'c', ...
	     stime, sx-sav,'y');
	 ttop = max([max(2*sd), max(sx-sav)]);
	 bbot = min([min(-2*sd), min(sx-sav)]);
     axis([-1000 1000 -5 45]);  %I added
      
      else
	 hl = plot(timebase, av+2*sd, 'c', ...
	     timebase, av-2*sd, 'c', stime, sx,'y');
	 ttop = max([max(av+2*sd), max(sx)]);
	 bbot = min([min(av-2*sd), min(sx)]);
      end;
      axis([-1000 1000 -5 45]);  %I added
      set(gca, 'FontSize', FontSize);
      
      %set(hl(3), 'Color', datacolor, 'LineWidth', datalwidth);
      set(hl(3), 'Color', datacolor, 'LineWidth', 4);
      if strcmp(sdstyle, 'none'),
	 delete(hl(1:2));
      else
	 set(hl(1:2), 'Color', sdcolor, 'LineStyle', sdstyle, ...
	     'LineWidth', sdlwidth);
      end;
      if ~nspks,
	 xlabel('time (ms)');
      else
	 tk = sum(numspikes(C1, from, to)+numspikes(C2, from, to));
	 xlabel(['time (ms)    nspks=' num2str(tk)]);
      end;

      switch normmode,
	 case 'hzy',
	 ylabel('Spikes^2/second');
      
	 case 'norm',
	 ylabel('Spikes/sec');
     axis([-1000 1000 -5 45]);  %I added
	 
	 case 'davey',
	 ylabel('Summed Counts'); 
      
	 case 'counts',
	 ylabel('Average counts');
      end;

      zoom on;
      title(sprintf('totspks = %d', ...
	  sum(numspikes(C1,from,to))+sum(numspikes(C2,from,to))));
        
      
      
      ax = axis;
      ax(1) = 1.1*timebase(1);
      ax(2) = 1.1*timebase(length(timebase));
      ax(3) = 1.1*bbot; if ax(3) > 0, ax(3) = 0; end;
      ax(4) = 1.1*ttop;
      if length(otimebase) == 1,
	 ax(1) = -binwidth/2;
	 ax(2) = binwidth/2;
      end;
      
      %axis(ax); %I commented out
      axis([-1000 1000 -5 45]); %I added
   
      axchilds = get(gca, 'Children');
      if ~all(zerocolor==-1),
	 zeroy = hlines(0);
	 set(zeroy, 'LineWidth', zlw*get(hl(3), 'LineWidth'), ...
	     'Color', zerocolor);
	 zerox = vlines(0);
	 set(zerox, 'LineWidth', zlw*get(hl(3), 'LineWidth'), ...
	     'Color', zerocolor);
	 set(gca, 'Children', [axchilds ; zeroy ; zerox]);
      end;
   end;
   
   
   if nargout > 0, xx = x; end;
   
