% rastermaker (C)		Plot rasters from a cell
%
%	rastermaker(C1, {C2})
%
% The plot is done on the current figure. C1 is a Sillito cell
% file as produced by rasterize. If the optional argument C2 is
% also present, the rasters for it are also plotted, but in a
% different color.
%
% An optional argument tail is the pair 'labels', M as in
%
%	rastermaker(C1, [C2, [C3]], 'labels', M)
%
% M is a matrix from which each row is interpreted as a string
% labeling the corresponding row in the C1 and C2 matrices. (C2
% need not be present.) This string is displayed as text together
% with the raster. If M is numeric but not a string, rastermaker
% automatically turns it into a string. 
%
% {'nocla'}   don't clear axes before drawing.
%


% type " nratermaker(S); " in command window to generate raster plot
function [] = rastermaker(varargin);

    %  ------------  various plotting parameters  -----------

	
    nrasters = -1;
    startraster = 1;

    from = 0;
    to = -Inf;
    lineheight = 0.7;
    linespacing = 0.2;
    yshift = 0;
    mode = 'lines';
    
    hardlims = 1;  % if 1, from and to are respected,otherwise not.
    
    endraster = startraster+nrasters-1;
    
    color1 = [244 164 96]/255;  % Sandy brown
    color2 = [112 255 76]/255;  % Lightish green
    %	color1 = 'red';
    %	color2 = 'blue';
    color3 = 'cyan';
    linewidth = get(gcf, 'DefaultLineLineWidth');
    
    % basecolor = [0.6 0.6 0.6]; % basecolor = 'k';
    basecolor = 0.1*[1 1 1];  % basecolor = -1;
    labels_color = 'white';
    label_every = 1;
    
    FontSize = get(gcf, 'DefaultAxesFontSize');
    Marker1 = '.';
    Marker2 = '.';
    Marker3 = '.';
    MarkerSize = 0;
    MarkerSize1 = 0;
    MarkerSize2 = 0;
    MarkerSize3 = 0;
    
    % color1 = 'yellow';
    % color2 = 'green';
    % color3 = 'cyan';
    % basecolor = 'magenta';
    % labels_color = 'white';
    
    C1 = []; C2 = []; C3 = [];
    
    % -----------  dealing with parsing the arguments  ---------
    
    have_labels = 0;
    nmatrices = 0;
    do_cla = 1;
    
    parseargs(varargin);
    
    if nrasters == -1, nrasters = rows(C1); end;
    endraster = startraster+nrasters-1;
    
    if nmatrices > 3, 
	error('sorry, can''t do more than three streams yet');
    end;
    
    maxrasters = max([rows(C1), rows(C2), rows(C3)]);
    if endraster > maxrasters,
	nrasters = maxrasters - startraster + 1;
	endraster = maxrasters;
    end;
    
    mint = min([mintime(C1, startraster, endraster), ...
	    mintime(C2, startraster, endraster), ...
	    mintime(C3, startraster, endraster)]);
    maxt = max([maxtime(C1, startraster, endraster), ...
	    maxtime(C2, startraster, endraster), ...
	    maxtime(C3, startraster, endraster)]);
    
    if to == -Inf, to = maxt; end;
    if to == from, to=from+1; end;
    

    if strcmp(mode, 'markers')  &  MarkerSize == 0,
       MarkerSize = 500/(nrasters*nmatrices);
    end;
    if MarkerSize1 == 0, MarkerSize1 = MarkerSize; end;
    if MarkerSize2 == 0, MarkerSize2 = MarkerSize; end;
    if MarkerSize3 == 0, MarkerSize3 = MarkerSize; end;
    
    lineheight = lineheight/nmatrices;
    linespacing = linespacing/nmatrices;
    
    
    % ----------  preparatory stuff  ------------------
    
    if ~hardlims,
	if mint > from,	from = mint-5; end; 
	if maxt < to,   to = maxt+5;   end;
    end;
    
    if do_cla, cla; end;
    axis([from, to, startraster-0.5, endraster+0.8]);


    %    -----------  actual procedure  -----------------

    if startraster > size(C1, 1)
	fprintf(2, 'rastermaker warning: ');
	fprintf(2, 'startraster out of range.\n');
    end;

    if startraster < 1
	fprintf(2, 'rastermaker: startraster < 1 ???\n'); return;
    end;

    xline = zeros(2, sum(C1(startraster:endraster,1))); 
    yline = zeros(2, sum(C1(startraster:endraster,1)));
    spcount = 0;
    for i=startraster:endraster,
       if i > rows(C1), break; end;
       if i ~= 1  &  basecolor ~= -1,
	  set(line([from,to],[i-0.5, i-0.5]), 'Color', basecolor);
       end;

       baseline = i-0.5 + yshift;

       if C1(i,1) > 0,
	  xline(1,spcount+1:spcount+C1(i,1)) = C1(i,2:C1(i,1)+1);
	  xline(2,spcount+1:spcount+C1(i,1)) = C1(i,2:C1(i,1)+1);
       
	  yline(1,spcount+1:spcount+C1(i,1)) = baseline;
	  yline(2,spcount+1:spcount+C1(i,1)) = baseline+lineheight;
       
	  spcount = spcount+C1(i,1);
       end;
    end;
    switch mode
       case 'lines',
       h = line(xline, yline, 'Color', color1, ...
	   'LineWidth', linewidth);
       
       case 'markers',
       yline(1,:) = yline(1,:)+lineheight/2;
       h = line(xline(1,:), yline(1,:), 'Color', color1, ...
	   'LineStyle', 'none', ...
	   'Marker', Marker1, 'MarkerSize', MarkerSize1);
    end;
    
       

    
    
    if nmatrices >= 2,
       xline = zeros(2, sum(C2(startraster:endraster,1))); 
       yline = zeros(2, sum(C2(startraster:endraster,1)));
       spcount = 0;
       for i=startraster:endraster,
	  if i > rows(C2), break; end;
	  baseline = i-0.5+lineheight+linespacing + yshift;
	  
	  if C2(i,1) > 0,
	     xline(1,spcount+1:spcount+C2(i,1)) =C2(i,2:C2(i,1)+1);
	     xline(2,spcount+1:spcount+C2(i,1)) =C2(i,2:C2(i,1)+1);
       
	     yline(1,spcount+1:spcount+C2(i,1)) = baseline;
	     yline(2,spcount+1:spcount+C2(i,1)) = ...
		 baseline+lineheight;
       
	     spcount = spcount+C2(i,1);
	  end;

       end;
       switch mode
	  case 'lines',
	  h = line(xline, yline, 'Color', color2, ...
	      'LineWidth', linewidth);
       
	  case 'markers',
	  yline(1,:) = yline(1,:)+lineheight/2;
	  h = line(xline(1,:), yline(1,:), 'Color', color2, ...
	      'LineStyle', 'none', ...
	      'Marker', Marker2, 'MarkerSize', MarkerSize2);
       end;
    end;
    
    if nmatrices >= 3,
       xline = zeros(2, sum(C3(startraster:endraster,1))); 
       yline = zeros(2, sum(C3(startraster:endraster,1)));
       spcount = 0;
       for i=startraster:endraster,
	  if i > rows(C3), break; end;
	  baseline = i-0.5+2*lineheight+2*linespacing + yshift;

	  if C3(i,1) > 0,
	     xline(1,spcount+1:spcount+C3(i,1)) =C3(i,2:C3(i,1)+1);
	     xline(2,spcount+1:spcount+C3(i,1)) =C3(i,2:C3(i,1)+1);
       
	     yline(1,spcount+1:spcount+C3(i,1)) = baseline;
	     yline(2,spcount+1:spcount+C3(i,1)) = ...
		 baseline+lineheight;
       
	     spcount = spcount+C3(i,1);
	  end;
       end;
       switch mode
	  case 'lines',
	  h = line(xline, yline, 'Color', color3, ...
	      'LineWidth', linewidth);
       
	  case 'markers',
	  yline(1,:) = yline(1,:)+lineheight/2;
	  h = line(xline(1,:), yline(1,:), 'Color', color3, ...
	      'LineStyle', 'none', ...
	      'Marker', Marker3, 'MarkerSize', MarkerSize3);
       end;
    end;
    
    xlabel('time (ms)', 'FontSize', FontSize);
    ylabel('Trial number', 'FontSize', FontSize);
    set(gca, 'FontSize', FontSize, 'Layer', 'top');
    
    %  ----------------  adding the labels  -----------------
    
    if have_labels
	label_x_pos = from + 0.85*(to-from);
	for i=startraster:endraster,
	    if i > size(labels, 1), break; end;
	    if rem(i, label_every) == 0,
	       h = text(label_x_pos, i, labels(i,:));
	       set(h, 'Color', labels_color, 'FontSize', FontSize);
	    end;
	end;
    end;

    
    zoom on;
    
    return;
    
% -------------------------------------------------------------


function [] = parseargs(arguments)
   
   nmatrices = 0;
   arg = 1; while arg <= length(arguments)
      
      if ~ischar(arguments{arg}),
	 nmatrices = nmatrices + 1;
	 assignin('caller', ...
	     ['C' num2str(nmatrices)], arguments{arg});
      else
      
	 switch arguments{arg}
	    
	    case { 'from', 'to', 'nrasters', 'startraster' },
	    if arg+1 <= length(arguments),
	       assignin('caller', arguments{arg},arguments{arg+1});
	       arg = arg+1;
	    end;
	    
	    case 'FontSize',
	    if arg+1 <= length(arguments),
	       assignin('caller', arguments{arg},arguments{arg+1});
	       arg = arg+1;
	    end;
	    
	    case {'linewidth', 'LineWidth'},
	    if arg+1 <= length(arguments),
	       assignin('caller', 'linewidth', arguments{arg+1});
	       arg = arg+1;
	    end;
	    
	    case 'yshift',
	    if arg+1 <= length(arguments),
	       assignin('caller', arguments{arg},arguments{arg+1});
	       arg = arg+1;
	    end;
	    
	    case { 'basecolor', 'color1', 'color2', 'color3' },
	    if arg+1 <= length(arguments),
	       assignin('caller', arguments{arg},arguments{arg+1});
	       arg = arg+1;
	    end;
	    
	    case { 'Marker1', 'Marker2', 'Marker3' },
	    if arg+1 <= length(arguments),
	       assignin('caller', arguments{arg},arguments{arg+1});
	       arg = arg+1;
	    end;
	    
	    case { 'lineheight', 'linespacing' },
	    if arg+1 <= length(arguments),
	       assignin('caller', arguments{arg},arguments{arg+1});
	       arg = arg+1;
	    end;
	    
	    case { 'MarkerSize', 'markersize' },
	    if arg+1 <= length(arguments),
	       assignin('caller', arguments{arg},arguments{arg+1});
	       arg = arg+1;
	    end;
	    
	    case { 'MarkerSize1', 'MarkerSize2', 'MarkerSize3' },
	    if arg+1 <= length(arguments),
	       assignin('caller', arguments{arg},arguments{arg+1});
	       arg = arg+1;
	    end;
	    
	    case 'markers',
	    assignin('caller', 'mode', 'markers');
	    
	    case 'lines',
	    assignin('caller', 'mode', 'lines');
	    
	    case {'nocla', 'no_cla'},
	    assignin('caller', 'do_cla', 0);
	    
	    case {'docla', 'do_cla'},
	    assignin('caller', 'do_cla', 1);
	    
	    case 'label_every',
	    if arg+1 <= length(arguments),
	       assignin('caller', arguments{arg},arguments{arg+1});
	       arg = arg+1;
	    end;
	    
	    case 'labels',
	    if arg+1 <= length(arguments),
	       labels = arguments{arg+1};
	       if ~ischar(labels), labels =colvec2str(labels); end;
	       assignin('caller', 'labels', labels);
	       assignin('caller', 'have_labels', 1);
	       arg = arg+1;
	    end;
	    
	    otherwise
	    
	 end;
      
      end;   

   arg = arg+1; end;
   
   assignin('caller', 'nmatrices', nmatrices);
   
   return;
   
   
