%zxcorr  [X, p1, p2, s1, s2] = zxcorr(C1, C2, nbins, from, to, ...
%                                      [rflag=0])
%
% Does raw cross-correlograms of Sillito-format data. Implemented
% as a mex-file (see zxcorr.c)
%
% Parameters:
% -----------
%
%   C1     Sillito format matrix
%
%   C2     Sillito format matrix
%
%   nbins  number of bins to to bin the spikes from each cell
%          with. Note that X will then be length (2*nbins)-1.
%
%   from   Ignore any spikes that happen before this time
%
%   to     Ignore any spikes that happen after this time
%
% 
% The binwidth will be set to ((to+1e-5)-from)/nbins. Binning is
% done before x-correlating.
%
% Returns:
% --------
%
%   X      The summed cross-correlogram of C1 and C2.
%         
%   p1     The summed psth of C1, going from 'from' to 'to'
%
%   p2     The summed psth of C2, going from 'from' to 'to'
%
%   s1     The summed binned squares of C1, 'from' to 'to'
%
%   s2     The summed binned squares of C2, 'from' to 'to'
% 
%
% None of the above are normalised by nrasters, they are all just
% the sums.
%
% An appropriate time axis to display the cross-correlogram would
% be (-(to-from)*(nbins-1)):binwidth:((to-from)*(nbins-1))
%



% The next part should only run if the mex-file is missing


function [X, p1, p2, s1, s2] = zxcorr(C1, C2, nbins, from, to, ...
                                       rflag)
   
   if nargin < 6, rflag = 0; end;
   
   binwidth = ((to+1e-5)-from)/nbins;
   nrasters = rows(C1);
   
   X = zeros(1, 2*nbins-1);
   p1 = zeros(1, nbins);
   p2 = zeros(1, nbins);
   s1 = zeros(1, nbins);
   s2 = zeros(1, nbins);
   
   for raster = 1:nrasters,
      
      v1 = zeros(1, nbins);
      v2 = zeros(1, nbins);
      
      for spike1=2:(C1(raster,1)+1),
	 time1 = C1(raster, spike1);
	 if time1 > to, break; end;
	 if time1 >= from,
	    bin1 = floor((time1-from)/binwidth) + 1;
	    p1(bin1) = p1(bin1)+1; v1(bin1) = v1(bin1) + 1;
	    
	    time2s = C2(raster,2:(C2(raster,1)+1));
	    time2s = time2s(find(from <= time2s & time2s <= to));
	    bin2s  = floor((time2s-from)/binwidth) + 1; 
%-------------------------------------------------------     
	    X = multisum(X, bin2s-bin1+nbins, 1);
%-------------------------------------------------------       
	    % for spike2 = 2:(C2(raster,1)+1),
	    % time2 = C2(raster, spike2);
	    % if time2 > to, break; end;
	    % if time2 >= from,
	    % bin2 = floor((time2-from)/binwidth) + 1;
	    % X(bin2-bin1+nbins) = X(bin2-bin1+nbins) + 1;
	    % end;
	    % end;
	 end;
      end;
      
      for spike2 = 2:(C2(raster,1)+1);
	 time2 = C2(raster, spike2);
	 if time2 > to, break; end;
	 if time2 >= from,
	    bin2 = floor((time2-from)/binwidth) + 1;
	    p2(bin2) = p2(bin2)+1; v2(bin2) = v2(bin2) + 1;
	 end;
      end;
      
      s1 = s1 + v1.^2;
      s2 = s2 + v2.^2;
   
      if rflag  &  rem(raster,20) == 0,
	 fprintf(2, 'Done raster %d\n', raster);
      end;
   
   end;
   
		  